{% extends "admin/change_form.html" %}

{% block after_field_sets %}
<div class="form-row field-req_body">
  <div>
    <label>Request body</label>
      <div class="request-body-decoded">{{ original.request_body_decoded }}</div>
    </div>
</div>
<div class="form-row field-res_body">
  <div>
    <label>Response body</label>
    <div class="response-body-decoded">{{ original.response_body_decoded }}</div>
  </div>
</div>
{% endblock %}
